@@ -4,7 +4,7 @@
 // Author:
 //   Gabriel Burt <gburt@novell.com>
 //
-// Copyright (C) 2007 Novell, Inc.
+// Copyright (C) 2007-2008 Novell, Inc.
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -31,7 +31,9 @@ using System.Collections.Generic;
 using Mono.Unix;
 using Gtk;
 
-using Banshee.Base;
+using Hyena;
+using Hyena.Widgets;
+
 using Banshee.Query;
 using Banshee.Sources;
 using Banshee.Library;
@@ -51,29 +53,33 @@ namespace Banshee.Gui
 
         private static readonly string [] require_selection_actions = new string [] {
             "TrackContextMenuAction", "TrackPropertiesAction", "AddToPlaylistAction",
-            "RemoveTracksAction", "RemoveTracksFromLibraryAction", "DeleteTracksFromDriveAction",
-            "RateTracksAction", "SelectNoneAction"
+            "RemoveTracksAction", "RemoveTracksFromLibraryAction", "OpenContainingFolderAction",
+            "DeleteTracksFromDriveAction", "RateTracksAction", "SelectNoneAction"
         };
-        
+
         public event EventHandler SelectionChanged;
 
-        public TrackActions (InterfaceActionService actionService) : base (actionService, "Track")
+        public TrackActions () : base ("Track")
         {
             Add (new ActionEntry [] {
-                new ActionEntry("TrackContextMenuAction", null, 
+                new ActionEntry("TrackContextMenuAction", null,
                     String.Empty, null, null, OnTrackContextMenu),
 
                 new ActionEntry("SelectAllAction", null,
                     Catalog.GetString("Select _All"), "<control>A",
                     Catalog.GetString("Select all tracks"), OnSelectAll),
-                    
+
                 new ActionEntry("SelectNoneAction", null,
                     Catalog.GetString("Select _None"), "<control><shift>A",
                     Catalog.GetString("Unselect all tracks"), OnSelectNone),
 
-                new ActionEntry ("TrackPropertiesAction", Stock.Edit,
+                new ActionEntry ("TrackEditorAction", Stock.Edit,
                     Catalog.GetString ("_Edit Track Information"), "E",
-                    Catalog.GetString ("Edit information on selected tracks"), OnTrackProperties),
+                    Catalog.GetString ("Edit information on selected tracks"), OnTrackEditor),
+
+                new ActionEntry ("TrackPropertiesAction", Stock.Properties,
+                    Catalog.GetString ("Properties"), null,
+                    Catalog.GetString ("View information on selected tracks"), OnTrackProperties),
 
                 new ActionEntry ("AddToPlaylistAction", null,
                     Catalog.GetString ("Add _to Playlist"), null,
@@ -93,6 +99,10 @@ namespace Banshee.Gui
                     Catalog.GetString ("Remove From _Library"), "",
                     Catalog.GetString ("Remove selected track(s) from library"), OnRemoveTracksFromLibrary),
 
+                new ActionEntry ("OpenContainingFolderAction", null,
+                    Catalog.GetString ("_Open Containing Folder"), null,
+                    Catalog.GetString ("Open the folder that contains the selected item"), OnOpenContainingFolder),
+
                 new ActionEntry ("DeleteTracksFromDriveAction", null,
                     Catalog.GetString ("_Delete From Drive"), null,
                     Catalog.GetString ("Permanently delete selected item(s) from medium"), OnDeleteTracksFromDrive),
@@ -130,14 +140,14 @@ namespace Banshee.Gui
                 current_source.TrackModel.Selection.Changed -= HandleSelectionChanged;
                 current_source = null;
             }
-            
+
             ITrackModelSource new_source = ActiveSource as ITrackModelSource;
             if (new_source != null) {
                 new_source.TrackModel.Selection.Changed += HandleSelectionChanged;
                 current_source = new_source;
             }
-            
-            UpdateActions ();
+
+            ThreadAssist.ProxyToMain (UpdateActions);
         }
 
         private void HandleActionsChanged (object sender, EventArgs args)
@@ -152,15 +162,17 @@ namespace Banshee.Gui
 
         private void HandleSelectionChanged (object sender, EventArgs args)
         {
-            OnSelectionChanged ();
-            UpdateActions ();
+            ThreadAssist.ProxyToMain (delegate {
+                OnSelectionChanged ();
+                UpdateActions ();
+            });
         }
 
         private void HandleEditMenuActivated (object sender, EventArgs args)
         {
             ResetRating ();
         }
-        
+
         private void OnSelectionChanged ()
         {
             EventHandler handler = SelectionChanged;
@@ -197,7 +209,7 @@ namespace Banshee.Gui
             Source source = ServiceManager.SourceManager.ActiveSource;
             bool in_database = source is DatabaseSource;
             PrimarySource primary_source = (source as PrimarySource) ?? (source.Parent as PrimarySource);
-            
+
             Hyena.Collections.Selection selection = (source is ITrackModelSource) ? (source as ITrackModelSource).TrackModel.Selection : null;
 
             if (selection != null) {
@@ -221,11 +233,24 @@ namespace Banshee.Gui
 
                     UpdateAction ("RemoveTracksAction", is_track_source && track_source.CanRemoveTracks, has_selection, source);
                     UpdateAction ("DeleteTracksFromDriveAction", is_track_source && track_source.CanDeleteTracks, has_selection, source);
+
+                    //if it can delete tracks, most likely it can open their folder
+                    UpdateAction ("OpenContainingFolderAction", is_track_source && track_source.CanDeleteTracks, has_single_selection, source);
+
                     UpdateAction ("RemoveTracksFromLibraryAction", source.Parent is LibrarySource, has_selection, null);
-                    
-                    UpdateAction ("TrackPropertiesAction", in_database, has_selection, source);
+
+                    UpdateAction ("TrackPropertiesAction", source.HasViewableTrackProperties, has_selection, source);
+                    UpdateAction ("TrackEditorAction", source.HasEditableTrackProperties, has_selection, source);
                     UpdateAction ("RateTracksAction", in_database, has_selection, null);
-                    UpdateAction ("AddToPlaylistAction", in_database && primary_source != null && primary_source.SupportsPlaylists, has_selection, null);
+                    UpdateAction ("AddToPlaylistAction", in_database && primary_source != null &&
+                            primary_source.SupportsPlaylists && !primary_source.PlaylistsReadOnly, has_selection, null);
+
+                    if (primary_source != null &&
+                        !(primary_source is LibrarySource) &&
+                        primary_source.StorageName != null) {
+                        this["DeleteTracksFromDriveAction"].Label = String.Format (
+                            Catalog.GetString ("_Delete From \"{0}\""), primary_source.StorageName);
+                    }
                 }
             } else {
                 Sensitive = Visible = false;
@@ -236,7 +261,7 @@ namespace Banshee.Gui
         {
             if (current_source != null) {
                 int rating = 0;
-    
+
                 // If there is only one track, get the preset rating
                 if (current_source.TrackModel.Selection.Count == 1) {
                     foreach (TrackInfo track in current_source.TrackModel.SelectedItems) {
@@ -248,7 +273,7 @@ namespace Banshee.Gui
         }
 
 #endregion
-            
+
 #region Action Handlers
 
         private void OnSelectAll (object o, EventArgs args)
@@ -269,19 +294,32 @@ namespace Banshee.Gui
             ShowContextMenu ("/TrackContextMenu");
         }
 
+        private bool RunSourceOverrideHandler (string sourceOverrideHandler)
+        {
+            Source source = current_source as Source;
+            InvokeHandler handler = source != null
+                ? source.GetInheritedProperty<InvokeHandler> (sourceOverrideHandler)
+                : null;
+
+            if (handler != null) {
+                handler ();
+                return true;
+            }
+
+            return false;
+        }
+
         private void OnTrackProperties (object o, EventArgs args)
         {
-            if (current_source != null) {
-                Source source = current_source as Source;
-                InvokeHandler handler = source != null 
-                    ? source.GetInheritedProperty<InvokeHandler> ("TrackPropertiesActionHandler") 
-                    : null;
-                
-                if (handler != null) {
-                    handler ();
-                } else {
-                    new TrackEditor (current_source.TrackModel.SelectedItems);
-                }
+            if (current_source != null && !RunSourceOverrideHandler ("TrackPropertiesActionHandler")) {
+                Banshee.Gui.TrackEditor.TrackEditorDialog.RunView (current_source.TrackModel);
+            }
+        }
+
+        private void OnTrackEditor (object o, EventArgs args)
+        {
+            if (current_source != null && !RunSourceOverrideHandler ("TrackEditorActionHandler")) {
+                Banshee.Gui.TrackEditor.TrackEditorDialog.RunEdit (current_source.TrackModel);
             }
         }
 
@@ -302,7 +340,7 @@ namespace Banshee.Gui
 
                 submenu.Append (this ["AddToNewPlaylistAction"].CreateMenuItem ());
                 bool separator_added = false;
-                
+
                 foreach (Source child in ActivePrimarySource.Children) {
                     PlaylistSource playlist = child as PlaylistSource;
                     if (playlist != null) {
@@ -310,7 +348,7 @@ namespace Banshee.Gui
                             submenu.Append (new SeparatorMenuItem ());
                             separator_added = true;
                         }
-                        
+
                         PlaylistMenuItem item = new PlaylistMenuItem (playlist);
                         item.Image = new Gtk.Image ("playlist-source", IconSize.Menu);
                         item.Activated += OnAddToExistingPlaylist;
@@ -318,7 +356,7 @@ namespace Banshee.Gui
                         submenu.Append (item);
                     }
                 }
-                
+
                 submenu.ShowAll ();
             }
         }
@@ -326,7 +364,7 @@ namespace Banshee.Gui
         private void OnAddToNewPlaylist (object o, EventArgs args)
         {
             // TODO generate name based on the track selection, or begin editing it
-            PlaylistSource playlist = new PlaylistSource ("New Playlist", ActivePrimarySource.DbId);
+            PlaylistSource playlist = new PlaylistSource (Catalog.GetString ("New Playlist"), ActivePrimarySource);
             playlist.Save ();
             playlist.PrimarySource.AddChildSource (playlist);
             ThreadAssist.SpawnFromMain (delegate {
@@ -373,6 +411,41 @@ namespace Banshee.Gui
             }
         }
 
+        private void OnOpenContainingFolder (object o, EventArgs args)
+        {
+            var source = ActiveSource as ITrackModelSource;
+
+            if (source == null ||
+                source.TrackModel == null ||
+                source.TrackModel.SelectedItems == null ||
+                source.TrackModel.SelectedItems.Count != 1) {
+                Log.Error ("Could not open containing folder");
+                return;
+            }
+
+            foreach (var track in source.TrackModel.SelectedItems) {
+                var path = System.IO.Path.GetDirectoryName (track.Uri.AbsolutePath);
+                if (Banshee.IO.Directory.Exists (path)) {
+                    System.Diagnostics.Process.Start (path);
+                    return;
+                }
+            }
+
+            var md = new HigMessageDialog (
+                ServiceManager.Get<GtkElementsService> ("GtkElementsService").PrimaryWindow,
+                DialogFlags.DestroyWithParent, MessageType.Warning,
+                ButtonsType.None, Catalog.GetString ("The folder could not be found."),
+                Catalog.GetString ("Please check that the track's location is accessible by the system.")
+            );
+            md.AddButton ("gtk-ok", ResponseType.Ok, true);
+
+            try {
+                md.Run ();
+            } finally {
+                md.Destroy ();
+            }
+        }
+
         private void OnDeleteTracksFromDrive (object o, EventArgs args)
         {
             ITrackModelSource source = ActiveSource as ITrackModelSource;
@@ -419,12 +492,12 @@ namespace Banshee.Gui
             if (!source.ConfirmRemoveTracks) {
                 return true;
             }
-            
+
             bool ret = false;
             string header = null;
             string message = null;
             string button_label = null;
-            
+
             if (delete) {
                 header = String.Format (
                     Catalog.GetPluralString (
@@ -444,15 +517,16 @@ namespace Banshee.Gui
                 );
                 button_label = "gtk-remove";
             }
-                
+
             HigMessageDialog md = new HigMessageDialog (
                 ServiceManager.Get<GtkElementsService> ("GtkElementsService").PrimaryWindow,
                 DialogFlags.DestroyWithParent, delete ? MessageType.Warning : MessageType.Question,
                 ButtonsType.None, header, message
             );
-            md.AddButton ("gtk-cancel", ResponseType.No, false);
-            md.AddButton (button_label, ResponseType.Yes, false);
-            
+            // Delete from Disk defaults to Cancel and the others to OK/Confirm.
+            md.AddButton ("gtk-cancel", ResponseType.No, delete);
+            md.AddButton (button_label, ResponseType.Yes, !delete);
+
             try {
                 if (md.Run () == (int) ResponseType.Yes) {
                     ret = true;
@@ -462,5 +536,11 @@ namespace Banshee.Gui
             }
             return ret;
         }
+
+        // Reserve strings in preparation for the 1.5.5 string freeze (BGO#611923)
+        public void ReservedStrings ()
+        {
+            Catalog.GetString ("View Track Information");
+        }
     }
 }
