@@ -36,26 +36,29 @@ using Banshee.Collection.Database;
 using Hyena;
 
 namespace Banshee.Dap.Ipod
-{   
+{
     public class IpodTrackInfo : DatabaseTrackInfo
     {
         private IPod.Track track;
         internal IPod.Track IpodTrack {
             get { return track; }
         }
-        
+
         private int ipod_id;
         internal int IpodId {
             get { return ipod_id; }
         }
-        
+
+        // Used for podcasts only
+        private string description;
+
         public IpodTrackInfo (IPod.Track track) : base ()
         {
             this.track = track;
             LoadFromIpodTrack ();
             CanSaveToDatabase = true;
         }
-        
+
         public IpodTrackInfo (TrackInfo track)
         {
             if (track is IpodTrackInfo) {
@@ -66,6 +69,7 @@ namespace Banshee.Dap.Ipod
                 AlbumTitle = track.AlbumTitle;
                 ArtistName = track.ArtistName;
                 BitRate = track.BitRate;
+                SampleRate = track.SampleRate;
                 Bpm = track.Bpm;
                 Comment = track.Comment;
                 Composer = track.Composer;
@@ -82,6 +86,7 @@ namespace Banshee.Dap.Ipod
                 LastPlayed = track.LastPlayed;
                 LastSkipped = track.LastSkipped;
                 PlayCount = track.PlayCount;
+                Rating = track.Rating;
                 ReleaseDate = track.ReleaseDate;
                 SkipCount = track.SkipCount;
                 TrackCount = track.TrackCount;
@@ -89,26 +94,33 @@ namespace Banshee.Dap.Ipod
                 TrackTitle = track.TrackTitle;
                 Year = track.Year;
                 MediaAttributes = track.MediaAttributes;
+
+                var podcast_info = track.ExternalObject as IPodcastInfo;
+                if (podcast_info != null) {
+                    description = podcast_info.Description;
+                    ReleaseDate = podcast_info.ReleaseDate;
+                }
             }
-            
+
             CanSaveToDatabase = true;
         }
-        
+
         private void LoadFromIpodTrack ()
         {
             try {
                 Uri = new SafeUri (track.Uri.LocalPath);
-            } catch { 
+            } catch {
                 Uri = null;
             }
 
             ExternalId = track.Id;
             ipod_id = (int)track.Id;
-            
+
             AlbumArtist = track.AlbumArtist;
             AlbumTitle = String.IsNullOrEmpty (track.Album) ? null : track.Album;
             ArtistName = String.IsNullOrEmpty (track.Artist) ? null : track.Artist;
             BitRate = track.BitRate;
+            SampleRate = track.SampleRate;
             Bpm = (int)track.BPM;
             Comment = track.Comment;
             Composer = track.Composer;
@@ -125,25 +137,26 @@ namespace Banshee.Dap.Ipod
             TrackCount = track.TotalTracks;
             TrackNumber = track.TrackNumber;
             TrackTitle = String.IsNullOrEmpty (track.Title) ? null : track.Title;
-            //ReleaseDate = track.DateReleased;
             Year = track.Year;
-            
+            description = track.Description;
+            ReleaseDate = track.DateReleased;
+
             switch (track.Rating) {
                 case IPod.TrackRating.One:   rating = 1; break;
                 case IPod.TrackRating.Two:   rating = 2; break;
                 case IPod.TrackRating.Three: rating = 3; break;
                 case IPod.TrackRating.Four:  rating = 4; break;
                 case IPod.TrackRating.Five:  rating = 5; break;
-                case IPod.TrackRating.Zero: 
+                case IPod.TrackRating.Zero:
                 default:                     rating = 0; break;
             }
-            
+
             if (track.IsProtected) {
                 PlaybackError = StreamPlaybackError.Drm;
             }
-            
+
             MediaAttributes = TrackMediaAttributes.AudioStream;
-            
+
             switch (track.Type) {
                 case IPod.MediaType.Audio:
                     MediaAttributes |= TrackMediaAttributes.Music;
@@ -173,7 +186,7 @@ namespace Banshee.Dap.Ipod
                     break;
             }
         }
-        
+
         public void CommitToIpod (IPod.Device device)
         {
             track = track ?? device.TrackDatabase.CreateTrack ();
@@ -185,9 +198,10 @@ namespace Banshee.Dap.Ipod
                 Log.Exception ("Failed to create System.Uri for iPod track", e);
                 device.TrackDatabase.RemoveTrack (track);
             }
-            
+
             track.AlbumArtist = AlbumArtist;
             track.BitRate = BitRate;
+            track.SampleRate = (ushort)SampleRate;
             track.BPM = (short)Bpm;
             track.Comment = Comment;
             track.Composer = Composer;
@@ -203,13 +217,13 @@ namespace Banshee.Dap.Ipod
             track.TotalTracks = TrackCount;
             track.TrackNumber = TrackNumber;
             track.Year = Year;
-            //track.DateReleased = ReleaseDate;
-            
+            track.DateReleased = ReleaseDate;
+
             track.Album = AlbumTitle;
             track.Artist = ArtistName;
             track.Title = TrackTitle;
             track.Genre = Genre;
-            
+
             switch (Rating) {
                 case 1: track.Rating = IPod.TrackRating.Zero; break;
                 case 2: track.Rating = IPod.TrackRating.Two; break;
@@ -220,12 +234,12 @@ namespace Banshee.Dap.Ipod
             }
 
             if (HasAttribute (TrackMediaAttributes.Podcast)) {
-                //track.Description = ..
-                //track.Category = ..
-                //track.RememberPosition = true;
-                //track.NotPlayedMark = track.PlayCount == 0;
+                track.DateReleased = ReleaseDate;
+                track.Description = description;
+                track.RememberPosition = true;
+                track.NotPlayedMark = track.PlayCount == 0;
             }
-            
+
             if (HasAttribute (TrackMediaAttributes.VideoStream)) {
                 if (HasAttribute (TrackMediaAttributes.Podcast)) {
                     track.Type = IPod.MediaType.VideoPodcast;
@@ -249,35 +263,35 @@ namespace Banshee.Dap.Ipod
                     track.Type = IPod.MediaType.Audio;
                 }
             }
-            
+
             if (CoverArtSpec.CoverExists (ArtworkId)) {
                 SetIpodCoverArt (device, track, CoverArtSpec.GetPath (ArtworkId));
             }
         }
-        
-        // FIXME: No reason for this to use GdkPixbuf - the file is on disk already in 
+
+        // FIXME: No reason for this to use GdkPixbuf - the file is on disk already in
         // the artwork cache as a JPEG, so just shove the bytes from disk into the track
-        
         public static void SetIpodCoverArt (IPod.Device device, IPod.Track track, string path)
         {
             try {
-                Gdk.Pixbuf pixbuf = new Gdk.Pixbuf (path);
+                Gdk.Pixbuf pixbuf = null;
+                foreach (IPod.ArtworkFormat format in device.LookupArtworkFormats (IPod.ArtworkUsage.Cover)) {
+                    if (!track.HasCoverArt (format)) {
+                        // Lazily load the pixbuf
+                        if (pixbuf == null) {
+                            pixbuf = new Gdk.Pixbuf (path);
+                        }
+
+                        track.SetCoverArt (format, IPod.ArtworkHelpers.ToBytes (format, pixbuf));
+                    }
+                }
+
                 if (pixbuf != null) {
-                    SetIpodCoverArt (device, track, IPod.ArtworkUsage.Cover, pixbuf);
                     pixbuf.Dispose ();
                 }
             } catch (Exception e) {
                 Log.Exception (String.Format ("Failed to set cover art on iPod from {0}", path), e);
             }
         }
-
-        private static void SetIpodCoverArt (IPod.Device device, IPod.Track track, IPod.ArtworkUsage usage, Gdk.Pixbuf pixbuf)
-        {
-            foreach (IPod.ArtworkFormat format in device.LookupArtworkFormats (usage)) {
-                if (!track.HasCoverArt (format)) {
-                    track.SetCoverArt (format, IPod.ArtworkHelpers.ToBytes (format, pixbuf));
-                }
-            }
-        }
     }
 }
