@@ -61,13 +61,13 @@ namespace Banshee.Dap.Mtp
         private Dictionary<string, Album> album_cache = new Dictionary<string, Album> ();
 
         private bool supports_jpegs = false;
-        private bool can_sync = NeverSyncAlbumArtSchema.Get () == false;
+        private bool can_sync_albumart = NeverSyncAlbumArtSchema.Get () == false;
         private int thumb_width = AlbumArtWidthSchema.Get ();
 
         public override void DeviceInitialize (IDevice device)
         {
             base.DeviceInitialize (device);
-            
+
             if (MediaCapabilities == null || !MediaCapabilities.IsType ("mtp")) {
                 throw new InvalidDeviceException ();
             }
@@ -109,6 +109,10 @@ namespace Banshee.Dap.Mtp
                     if (mtp_serial.Contains (device.Serial)) {
                         mtp_device = devices[0];
                         mtp_source = this;
+                    } else if (device.Serial.Contains (mtp_serial.TrimStart('0'))) {
+                        // Special case for sony walkman players; BGO #543938
+                        mtp_device = devices[0];
+                        mtp_source = this;
                     }
                 }
 
@@ -143,25 +147,41 @@ namespace Banshee.Dap.Mtp
 
             AddDapProperty (Catalog.GetString ("Serial number"), mtp_device.SerialNumber);
             AddDapProperty (Catalog.GetString ("Version"), mtp_device.Version);
-            AddDapProperty (Catalog.GetString ("Battery level"), String.Format ("{0:0%}", mtp_device.BatteryLevel/100.0));
+            try {
+                AddDapProperty (Catalog.GetString ("Battery level"), String.Format ("{0:0%}", mtp_device.BatteryLevel/100.0));
+            } catch (Exception e) {
+                Log.Exception ("Unable to get battery level from MTP device", e);
+            }
         }
 
         protected override void LoadFromDevice ()
         {
             track_map = new Dictionary<int, Track> ();
             try {
-                List<Track> files = mtp_device.GetAllTracks (delegate (ulong current, ulong total, IntPtr data) {
-                    //user_event.Progress = (double)current / total;
-                    // Translators: {0} is the name of the MTP audio device (eg Gabe's Zen Player), {1} is the
-                    // track currently being loaded, and {2} is the total # of tracks that will be loaded.
-                    SetStatus (String.Format (Catalog.GetString ("Loading {0} - {1} of {2}"), Name, current, total), false);
-                    return 0;
-                });
+                List<Track> files = null;
+                lock (mtp_device) {
+                    files = mtp_device.GetAllTracks (delegate (ulong current, ulong total, IntPtr data) {
+                        //user_event.Progress = (double)current / total;
+                        // Translators: {0} is the name of the MTP audio device (eg Gabe's Zen Player), {1} is the
+                        // track currently being loaded, and {2} is the total # of tracks that will be loaded.
+                        SetStatus (String.Format (Catalog.GetString ("Loading {0} - {1} of {2}"), Name, current, total), false);
+                        return 0;
+                    });
+                }
 
                 /*if (user_event.IsCancelRequested) {
                     return;
                 }*/
 
+                // Delete any empty albums
+                lock (mtp_device) {
+                    foreach (Album album in mtp_device.GetAlbums ()) {
+                        if (album.Count == 0) {
+                            album.Remove ();
+                        }
+                    }
+                }
+
                 int [] source_ids = new int [] { DbId };
                 foreach (Track mtp_track in files) {
                     int track_id;
@@ -178,15 +198,18 @@ namespace Banshee.Dap.Mtp
                 Hyena.Data.Sqlite.HyenaSqliteCommand insert_cmd = new Hyena.Data.Sqlite.HyenaSqliteCommand (
                     @"INSERT INTO CorePlaylistEntries (PlaylistID, TrackID)
                         SELECT ?, TrackID FROM CoreTracks WHERE PrimarySourceID = ? AND ExternalID = ?");
-                foreach (MTP.Playlist playlist in mtp_device.GetPlaylists ()) {
-                    PlaylistSource pl_src = new PlaylistSource (playlist.Name, this);
-                    pl_src.Save ();
-                    // TODO a transaction would make sense here (when the threading issue is fixed)
-                    foreach (int id in playlist.TrackIds) {
-                        ServiceManager.DbConnection.Execute (insert_cmd, pl_src.DbId, this.DbId, id);
+
+                lock (mtp_device) {
+                    foreach (MTP.Playlist playlist in mtp_device.GetPlaylists ()) {
+                        PlaylistSource pl_src = new PlaylistSource (playlist.Name, this);
+                        pl_src.Save ();
+                        // TODO a transaction would make sense here (when the threading issue is fixed)
+                        foreach (int id in playlist.TrackIds) {
+                            ServiceManager.DbConnection.Execute (insert_cmd, pl_src.DbId, this.DbId, id);
+                        }
+                        pl_src.UpdateCounts ();
+                        AddChildSource (pl_src);
                     }
-                    pl_src.UpdateCounts ();
-                    AddChildSource (pl_src);
                 }
 
             } catch (Exception e) {
@@ -221,15 +244,15 @@ namespace Banshee.Dap.Mtp
                     playlist.Remove ();
                 }
                 device_playlists.Clear ();
-    
+
                 // Add playlists from Banshee to the device
                 foreach (Source child in Children) {
                     PlaylistSource from = child as PlaylistSource;
                     if (from != null && from.Count > 0) {
                         MTP.Playlist playlist = new MTP.Playlist (mtp_device, from.Name);
                         foreach (int track_id in ServiceManager.DbConnection.QueryEnumerable<int> (String.Format (
-                            "SELECT CoreTracks.ExternalID FROM CoreTracks{0} WHERE {1}",
-                            from.DatabaseTrackModel.JoinFragment, from.DatabaseTrackModel.Condition)))
+                            "SELECT CoreTracks.ExternalID FROM {0} WHERE {1}",
+                            from.DatabaseTrackModel.ConditionFromFragment, from.DatabaseTrackModel.Condition)))
                         {
                             playlist.AddTrack (track_id);
                         }
@@ -272,27 +295,31 @@ namespace Banshee.Dap.Mtp
             }
         }
 
+        private long bytes_used;
         public override long BytesUsed {
             get {
-                long count = 0;
-                lock (mtp_device) {
+                if (Monitor.TryEnter (mtp_device)) {
+                    bytes_used = 0;
                     foreach (DeviceStorage s in mtp_device.GetStorage ()) {
-                        count += (long) s.MaxCapacity - (long) s.FreeSpaceInBytes;
+                        bytes_used += (long) s.MaxCapacity - (long) s.FreeSpaceInBytes;
                     }
+                    Monitor.Exit (mtp_device);
                 }
-                return count;
+                return bytes_used;
             }
         }
-        
+
+        private long bytes_capacity;
         public override long BytesCapacity {
             get {
-                long count = 0;
-                lock (mtp_device) {
+                if (Monitor.TryEnter (mtp_device)) {
+                    bytes_capacity = 0;
                     foreach (DeviceStorage s in mtp_device.GetStorage ()) {
-                        count += (long) s.MaxCapacity;
+                        bytes_capacity += (long) s.MaxCapacity;
                     }
+                    Monitor.Exit (mtp_device);
                 }
-                return count;
+                return bytes_capacity;
             }
         }
 
@@ -305,43 +332,48 @@ namespace Banshee.Dap.Mtp
             if (track.PrimarySourceId == DbId)
                 return;
 
-            Track mtp_track = TrackInfoToMtpTrack (track, fromUri);
-            bool video = (track.MediaAttributes & TrackMediaAttributes.VideoStream) != 0;
             lock (mtp_device) {
+                Track mtp_track = TrackInfoToMtpTrack (track, fromUri);
+                bool video = track.HasAttribute (TrackMediaAttributes.VideoStream);
                 mtp_device.UploadTrack (fromUri.LocalPath, mtp_track, GetFolderForTrack (track), OnUploadProgress);
-            }
 
-            // Add/update album art
-            if (!video) {
-                string key = MakeAlbumKey (track.ArtistName, track.AlbumTitle);
-                if (!album_cache.ContainsKey (key)) {
-                    Album album = new Album (mtp_device, track.AlbumTitle, track.ArtistName, track.Genre);
-                    album.AddTrack (mtp_track);
+                // Add/update album art
+                if (!video) {
+                    string key = MakeAlbumKey (track.AlbumArtist, track.AlbumTitle);
+                    if (!album_cache.ContainsKey (key)) {
+                        Album album = new Album (mtp_device, track.AlbumTitle, track.AlbumArtist, track.Genre, track.Composer);
+                        album.AddTrack (mtp_track);
 
-                    if (supports_jpegs && can_sync) {
-                        try {
-                            Gdk.Pixbuf pic = ServiceManager.Get<Banshee.Collection.Gui.ArtworkManager> ().LookupScalePixbuf (
-                                track.ArtworkId, thumb_width
-                            );
-                            if (pic != null) {
-                                byte [] bytes = pic.SaveToBuffer ("jpeg");
-                                album.Save (bytes, (uint)pic.Width, (uint)pic.Height);
-                                Banshee.Collection.Gui.ArtworkManager.DisposePixbuf (pic);
+                        if (supports_jpegs && can_sync_albumart) {
+                            try {
+                                Gdk.Pixbuf pic = ServiceManager.Get<Banshee.Collection.Gui.ArtworkManager> ().LookupScalePixbuf (
+                                    track.ArtworkId, thumb_width
+                                );
+                                if (pic != null) {
+                                    byte [] bytes = pic.SaveToBuffer ("jpeg");
+                                    album.Save (bytes, (uint)pic.Width, (uint)pic.Height);
+                                    Banshee.Collection.Gui.ArtworkManager.DisposePixbuf (pic);
+                                }
+                                album_cache[key] = album;
+                            } catch (Exception e) {
+                                Log.Debug ("Failed to create MTP Album", e.Message);
                             }
+                        } else {
+                            album.Save ();
                             album_cache[key] = album;
-                        } catch {}
+                        }
+                    } else {
+                        Album album = album_cache[key];
+                        album.AddTrack (mtp_track);
+                        album.Save ();
                     }
-                } else {
-                    Album album = album_cache[key];
-                    album.AddTrack (mtp_track);
-                    album.Save ();
                 }
-            }
 
-            MtpTrackInfo new_track = new MtpTrackInfo (mtp_device, mtp_track);
-            new_track.PrimarySource = this;
-            new_track.Save (false);
-            track_map[new_track.TrackId] = mtp_track;
+                MtpTrackInfo new_track = new MtpTrackInfo (mtp_device, mtp_track);
+                new_track.PrimarySource = this;
+                new_track.Save (false);
+                track_map[new_track.TrackId] = mtp_track;
+            }
         }
 
         private Folder GetFolderForTrack (TrackInfo track)
@@ -380,7 +412,7 @@ namespace Banshee.Dap.Mtp
                         album_cache.Remove (key);
                     }
                 }
-                
+
                 return true;
             }
         }
@@ -418,9 +450,9 @@ namespace Banshee.Dap.Mtp
             Dispose ();
         }
 
-        private static string MakeAlbumKey (string artist, string album)
+        private static string MakeAlbumKey (string album_artist, string album)
         {
-            return String.Format ("{0}_{1}", artist, album);
+            return String.Format ("{0}_{1}", album_artist, album);
         }
 
         public static readonly SchemaEntry<bool> NeverSyncAlbumArtSchema = new SchemaEntry<bool>(
