@@ -268,8 +268,7 @@ namespace Hyena.Widgets
                 State == StateType.Insensitive ? 1 : 0.90,
                 State == StateType.Insensitive ? 1 : 0.55,
                 State == StateType.Insensitive ? 1 : 0.45);
-            ((IDisposable)cr.Target).Dispose ();
-            ((IDisposable)cr).Dispose ();
+            CairoExtensions.DisposeContext (cr);
 
             return true;
         }
@@ -388,6 +387,7 @@ namespace Hyena.Widgets
 
 #region Test Module
 
+#if ENABLE_ATK
     public class RatingAccessible : Atk.Object, Atk.Value, Atk.ValueImplementor
     {
         private RatingEntry rating;
@@ -436,6 +436,7 @@ namespace Hyena.Widgets
             return true;
         }
     }
+#endif
 
 #if ENABLE_ATK
     internal class RatingAccessibleFactory : Atk.ObjectFactory
@@ -495,6 +496,7 @@ namespace Hyena.Widgets
             Pango.FontDescription fd = entry3.PangoContext.FontDescription.Copy ();
             fd.Size = (int)(fd.Size * Pango.Scale.XXLarge);
             entry3.ModifyFont (fd);
+            fd.Dispose ();
             box.PackStart (entry3, true, true, 0);
 
             pbox.ShowAll ();
