@@ -39,11 +39,12 @@ namespace Hyena.Data.Sqlite
         private readonly List<DatabaseColumn> columns = new List<DatabaseColumn> ();
         private readonly List<DatabaseColumn> select_columns = new List<DatabaseColumn> ();
         private readonly List<VirtualDatabaseColumn> virtual_columns = new List<VirtualDatabaseColumn> ();
-        
+
         private DatabaseColumn key;
         private int key_select_column_index;
         private HyenaSqliteConnection connection;
-        
+        private bool check_table = true;
+
         private HyenaSqliteCommand create_command;
         private HyenaSqliteCommand insert_command;
         private HyenaSqliteCommand update_command;
@@ -51,13 +52,13 @@ namespace Hyena.Data.Sqlite
         private HyenaSqliteCommand select_command;
         private HyenaSqliteCommand select_range_command;
         private HyenaSqliteCommand select_single_command;
-        
+
         private string table_name;
         private string primary_key;
         private string select;
         private string from;
         private string where;
-        
+
         private const string HYENA_DATABASE_NAME = "hyena_database_master";
 
         public virtual string TableName { get { return table_name; } }
@@ -78,23 +79,28 @@ namespace Hyena.Data.Sqlite
         {
             return new T ();
         }
-        
+
         protected virtual string HyenaTableName {
             get { return "HyenaModelVersions"; }
         }
-        
+
         public HyenaSqliteConnection Connection {
             get { return connection; }
         }
-        
+
         protected SqliteModelProvider (HyenaSqliteConnection connection)
         {
             this.connection = connection;
         }
-        
-        public SqliteModelProvider (HyenaSqliteConnection connection, string table_name) : this (connection)
+
+        public SqliteModelProvider (HyenaSqliteConnection connection, string table_name) : this (connection, table_name, true)
+        {
+        }
+
+        public SqliteModelProvider (HyenaSqliteConnection connection, string table_name, bool checkTable) : this (connection)
         {
             this.table_name = table_name;
+            this.check_table = checkTable;
             Init ();
         }
 
@@ -123,13 +129,15 @@ namespace Hyena.Data.Sqlite
             if (key == null) {
                 throw new Exception (String.Format ("The {0} table does not have a primary key", TableName));
             }
-            
+
             key_select_column_index = select_columns.IndexOf (key);
-            
-            CheckVersion ();
-            CheckTable ();
+
+            if (check_table) {
+                CheckVersion ();
+                CheckTable ();
+            }
         }
-        
+
         protected virtual void CheckVersion ()
         {
             if (connection.TableExists (HyenaTableName)) {
@@ -158,33 +166,33 @@ namespace Hyena.Data.Sqlite
                         version INTEGER)",
                     HyenaTableName)
                 );
-                
+
                 InsertVersion (HYENA_DATABASE_NAME, DatabaseVersion);
                 InsertVersion (TableName, ModelVersion);
             }
         }
-        
+
         private string SelectVersionSql (string name)
         {
             return String.Format (
                 "SELECT version FROM {0} WHERE name='{1}'",
                 HyenaTableName, name);
         }
-        
+
         private void UpdateVersion (string name, int version)
         {
             connection.Execute (String.Format (
                 "UPDATE {0} SET version={1} WHERE name='{2}'",
                 HyenaTableName, version, name));
         }
-        
+
         private void InsertVersion (string name, int version)
         {
             connection.Execute (String.Format (
                 "INSERT INTO {0} (name, version) VALUES ('{1}', {2})",
                 HyenaTableName, name, version));
         }
-        
+
         protected void CheckTable ()
         {
             //Console.WriteLine ("In {0} checking for table {1}", this, TableName);
@@ -205,7 +213,7 @@ namespace Hyena.Data.Sqlite
                 CreateTable ();
             }
         }
-        
+
         private void AddColumn (MemberInfo member, Attribute attribute)
         {
             DatabaseColumnAttribute column = attribute as DatabaseColumnAttribute;
@@ -213,7 +221,7 @@ namespace Hyena.Data.Sqlite
                 DatabaseColumn c = member is FieldInfo
                     ? new DatabaseColumn ((FieldInfo)member, column)
                     : new DatabaseColumn ((PropertyInfo)member, column);
-                    
+
                 AddColumn (c, column.Select);
             }
             VirtualDatabaseColumnAttribute virtual_column = attribute as VirtualDatabaseColumnAttribute;
@@ -225,7 +233,7 @@ namespace Hyena.Data.Sqlite
                 }
             }
         }
-        
+
         protected void AddColumn (DatabaseColumn c, bool select)
         {
             foreach (DatabaseColumn col in columns) {
@@ -242,13 +250,13 @@ namespace Hyena.Data.Sqlite
                     );
                 }
             }
-            
+
             columns.Add (c);
 
             if (select) {
                 select_columns.Add (c);
             }
-            
+
             if ((c.Constraints & DatabaseColumnConstraints.PrimaryKey) > 0) {
                 if (key != null) {
                     throw new Exception (String.Format (
@@ -258,7 +266,7 @@ namespace Hyena.Data.Sqlite
                 key = c;
             }
         }
-        
+
         protected virtual void CreateTable ()
         {
             connection.Execute (CreateCommand);
@@ -271,7 +279,7 @@ namespace Hyena.Data.Sqlite
                 }
             }
         }
-        
+
         protected void CreateIndex (string name, string columns)
         {
             Connection.Execute (String.Format (
@@ -279,22 +287,28 @@ namespace Hyena.Data.Sqlite
                 name, TableName, columns
             ));
         }
-        
-        public virtual void Save (T target)
+
+        public virtual void Save (T target, bool force_insert)
         {
             try {
-                if (Convert.ToInt32 (key.GetRawValue (target)) > 0) {
+                if (Convert.ToInt32 (key.GetRawValue (target)) > 0 && !force_insert) {
                     Update (target);
                 } else {
                     key.SetValue (target, Insert (target));
                 }
             } catch (Exception e) {
-                Hyena.Log.Exception (e); 
+                Hyena.Log.Exception (e);
                 Hyena.Log.DebugFormat ("type of key value: {0}", key.GetRawValue (target).GetType ());
                 throw;
             }
+
         }
-        
+
+        public virtual void Save (T target)
+        {
+            Save (target, false);
+        }
+
         protected virtual object [] GetInsertParams (T target)
         {
             // TODO create an instance variable object array and reuse it? beware threading issues
@@ -308,7 +322,7 @@ namespace Hyena.Data.Sqlite
             }
             return values;
         }
-        
+
         protected int Insert (T target)
         {
             return connection.Execute (InsertCommand, GetInsertParams (target));
@@ -328,30 +342,30 @@ namespace Hyena.Data.Sqlite
             values[j] = key.GetValue (target);
             return values;
         }
-        
+
         protected void Update (T target)
         {
             connection.Execute (UpdateCommand, GetUpdateParams (target));
         }
-        
+
         public virtual T Load (IDataReader reader)
         {
             T item = MakeNewObject ();
             Load (reader, item);
             return item;
         }
-        
+
         public void Load (IDataReader reader, T target)
         {
             int i = 0;
-            
+
             AbstractDatabaseColumn bad_column = null;
             try {
                 foreach (DatabaseColumn column in select_columns) {
                     bad_column = column;
                     column.SetValue (target, reader, i++);
                 }
-                
+
                 foreach (VirtualDatabaseColumn column in virtual_columns) {
                     bad_column = column;
                     column.SetValue (target, reader, i++);
@@ -363,7 +377,7 @@ namespace Hyena.Data.Sqlite
                 );
             }
         }
-        
+
         public IEnumerable<T> FetchAll ()
         {
             using (IDataReader reader = connection.Query (SelectCommand)) {
@@ -380,17 +394,22 @@ namespace Hyena.Data.Sqlite
             }
             return default(T);
         }
-        
+
         public IEnumerable<T> FetchAllMatching (string condition, params object [] vals)
         {
-            HyenaSqliteCommand fetch_matching_command = new HyenaSqliteCommand (String.Format ("{0} AND {1}", SelectCommand.Text, condition));
+            HyenaSqliteCommand fetch_matching_command = CreateFetchCommand (condition);
             using (IDataReader reader = connection.Query (fetch_matching_command, vals)) {
                 while (reader.Read ()) {
                     yield return Load (reader);
                 }
             }
         }
-        
+
+        public HyenaSqliteCommand CreateFetchCommand (string condition)
+        {
+            return new HyenaSqliteCommand (String.Format ("{0} AND {1}", SelectCommand.Text, condition));
+        }
+
         public IEnumerable<T> FetchRange (int offset, int limit)
         {
             using (IDataReader reader = connection.Query (SelectRangeCommand, offset, limit)) {
@@ -399,12 +418,12 @@ namespace Hyena.Data.Sqlite
                 }
             }
         }
-        
+
         public T FetchSingle (int id)
         {
             return FetchSingle ((long) id);
         }
-        
+
         public virtual T FetchSingle (long id)
         {
             using (IDataReader reader = connection.Query (SelectSingleCommand, id)) {
@@ -414,28 +433,33 @@ namespace Hyena.Data.Sqlite
             }
             return default(T);
         }
-        
+
         protected long PrimaryKeyFor (T item)
         {
             return Convert.ToInt64 (key.GetValue (item));
         }
-        
+
         protected long PrimaryKeyFor (IDataReader reader)
         {
             return Convert.ToInt64 (reader[key_select_column_index]);
         }
-        
+
         public virtual void Delete (long id)
         {
             if (id > 0)
                 connection.Execute (DeleteCommand, id);
         }
-        
+
         public void Delete (T item)
         {
             Delete (PrimaryKeyFor (item));
         }
-        
+
+        public void Delete (string condition, params object [] vals)
+        {
+            connection.Execute (String.Format ("DELETE FROM {0} WHERE {1}", TableName, condition), vals);
+        }
+
         public virtual void Delete (IEnumerable<T> items)
         {
             List<long> ids = new List<long> ();
@@ -445,7 +469,7 @@ namespace Hyena.Data.Sqlite
                 if (id > 0)
                     ids.Add (id);
             }
-            
+
             if (ids.Count > 0)
                 connection.Execute (DeleteCommand, ids.ToArray ());
         }
@@ -475,8 +499,11 @@ namespace Hyena.Data.Sqlite
                     column.SetValue (copy, column.GetRawValue (original));
                 }
             }
+            foreach (var column in virtual_columns) {
+                column.SetValue (copy, column.GetRawValue (original));
+            }
         }
-        
+
         protected virtual HyenaSqliteCommand CreateCommand {
             get {
                 if (create_command == null) {
@@ -499,7 +526,7 @@ namespace Hyena.Data.Sqlite
                 return create_command;
             }
         }
-        
+
         protected virtual HyenaSqliteCommand InsertCommand {
             get {
                 // FIXME can this string building be done more nicely?
@@ -528,7 +555,7 @@ namespace Hyena.Data.Sqlite
                 return insert_command;
             }
         }
-        
+
         protected virtual HyenaSqliteCommand UpdateCommand {
             get {
                 if (update_command == null) {
@@ -556,7 +583,7 @@ namespace Hyena.Data.Sqlite
                 return update_command;
             }
         }
-        
+
         protected virtual HyenaSqliteCommand SelectCommand {
             get {
                 if (select_command == null) {
@@ -570,11 +597,11 @@ namespace Hyena.Data.Sqlite
                 return select_command;
             }
         }
-        
+
         protected virtual HyenaSqliteCommand SelectRangeCommand {
             get {
                 if (select_range_command == null) {
-                    select_command = new HyenaSqliteCommand (
+                    select_range_command = new HyenaSqliteCommand (
                         String.Format (
                             "SELECT {0} FROM {1}{2}{3} LIMIT ?, ?",
                             Select, From,
@@ -586,7 +613,7 @@ namespace Hyena.Data.Sqlite
                 return select_range_command;
             }
         }
-        
+
         protected virtual HyenaSqliteCommand SelectSingleCommand {
             get {
                 if (select_single_command == null) {
@@ -602,7 +629,7 @@ namespace Hyena.Data.Sqlite
                 return select_single_command;
             }
         }
-        
+
         protected virtual HyenaSqliteCommand DeleteCommand {
             get {
                 if (delete_command == null) {
@@ -613,7 +640,7 @@ namespace Hyena.Data.Sqlite
                 return delete_command;
             }
         }
-        
+
         public virtual string Select {
             get {
                 if (select == null) {
@@ -622,7 +649,7 @@ namespace Hyena.Data.Sqlite
                 return select;
             }
         }
-        
+
         public virtual string From {
             get {
                 if (from == null) {
@@ -631,7 +658,7 @@ namespace Hyena.Data.Sqlite
                 return from;
             }
         }
-        
+
         public virtual string Where {
             get {
                 if (where == null) {
@@ -640,7 +667,7 @@ namespace Hyena.Data.Sqlite
                 return where;
             }
         }
-        
+
         public string PrimaryKey {
             get {
                 if (primary_key == null) {
@@ -650,7 +677,7 @@ namespace Hyena.Data.Sqlite
             }
             protected set { primary_key = value; }
         }
-        
+
         private void BuildQuerySql ()
         {
             StringBuilder select_builder = new StringBuilder ();
@@ -665,38 +692,41 @@ namespace Hyena.Data.Sqlite
                 select_builder.Append ('.');
                 select_builder.Append (column.Name);
             }
-            
+
             StringBuilder where_builder = new StringBuilder ();
             Dictionary<string, string> tables = new Dictionary<string,string> (virtual_columns.Count + 1);
             bool first_virtual = true;
             foreach (VirtualDatabaseColumn column in virtual_columns) {
-                if (first_virtual) {
-                    first_virtual = false;
-                } else {
-                    where_builder.Append (" AND ");
-                }
                 if (first) {
                     first = false;
                 } else {
                     select_builder.Append (',');
                 }
+
                 select_builder.Append (column.TargetTable);
                 select_builder.Append ('.');
                 select_builder.Append (column.Name);
-                
-                where_builder.Append (column.TargetTable);
-                where_builder.Append ('.');
-                where_builder.Append (column.ForeignKey);
-                where_builder.Append (" = ");
-                where_builder.Append (TableName);
-                where_builder.Append ('.');
-                where_builder.Append (column.LocalKey);
-                
-                if (!tables.ContainsKey (column.TargetTable)) {
+
+                bool table_not_joined = !tables.ContainsKey (column.TargetTable);
+                if (first_virtual) {
+                    first_virtual = false;
+                } else if (table_not_joined) {
+                    where_builder.Append (" AND ");
+                }
+
+                if (table_not_joined) {
+                    where_builder.Append (column.TargetTable);
+                    where_builder.Append ('.');
+                    where_builder.Append (column.ForeignKey);
+                    where_builder.Append (" = ");
+                    where_builder.Append (TableName);
+                    where_builder.Append ('.');
+                    where_builder.Append (column.LocalKey);
+
                     tables.Add (column.TargetTable, null);
                 }
             }
-            
+
             StringBuilder from_builder = new StringBuilder ();
             from_builder.Append (TableName);
             foreach (KeyValuePair<string, string> pair in tables) {
@@ -708,27 +738,27 @@ namespace Hyena.Data.Sqlite
             from = from_builder.ToString ();
             where = where_builder.ToString ();
         }
-        
+
         public U GetProperty <U> (T item, DbColumn column)
         {
             CheckProperty (typeof (U), column);
-            
+
             return connection.Query<U> (String.Format (
                 "SELECT {0} FROM {1} WHERE {2}={3}",
                 column.Name, TableName, key.Name, key.GetValue (item)));
         }
-        
+
         public void SetProperty <U> (T item, U value, DbColumn column)
         {
             CheckProperty (typeof (U), column);
-            
+
             connection.Execute (String.Format (
                 "UPDATE {0} SET {1}='{2}' WHERE {3}={4}",
                 TableName, column.Name,
                 SqliteUtils.ToDbFormat (typeof (U), value),
                 key.Name, key.GetValue (item)));
         }
-        
+
         public void ClearProperty <U> (DbColumn column)
         {
             if (!connection.ColumnExists (TableName, column.Name)) {
@@ -741,7 +771,7 @@ namespace Hyena.Data.Sqlite
                     TableName, column.Name, column.DefaultValue));
             }
         }
-        
+
         private void CheckProperty (Type type, DbColumn column)
         {
             if (!connection.ColumnExists (TableName, column.Name)) {
@@ -750,7 +780,7 @@ namespace Hyena.Data.Sqlite
                     column.Name, column.DefaultValue, column.Constraints));
             }
         }
-                                              
+
         private void AddColumnToTable (string column_schema)
         {
             connection.Execute (String.Format (
